# -*- coding: utf-8 -*-
# Warteschlange_mit_Iterator.py
'''Das Projekt implementiert objektorientiert
eine Warteschlange mit einem Iterator,
intern realisiert mit einer einfach verketteten Liste'''

class Warteschlange(object) :
    '''Implementiert eine Warteschlange mit Iterator'''

    def __init__(self) :
        '''Konstruktor'''
        self.__kopf = None
        self.__schwanz = None

    def __iter__(self):
        '''initialisiert den Iterator'''
        self.aktuell=self.__kopf
        return self

    def __next__(self):
        '''definiert den Iterationsschritt'''
        if self.aktuell==None:
            raise StopIteration
        else:
            wert=self.aktuell.GibInhalt()
            self.aktuell=self.aktuell.GibNachfolger()
            return wert
            
    def Anstellen (self, objekt) :
        '''hängt ein neues Element am Ende der Warteschlange an'''
        neu = Element_WS(objekt)
        if self.__kopf == None:
            self.__kopf = neu
            self.__schwanz = neu
        else:
            self.__schwanz.SetzeNachfolger(neu)
            self.__schwanz = neu

    def Aufrufen (self) :
        '''setzt den Kopf der Warteschlange weiter und
        gibt den Inhalt des bisherigen Kopfes zurück'''
        if self.__kopf == None:
            return None
        kopfObjekt = self.DerNaechsteIst()
        self.__kopf = self.__kopf.GibNachfolger()
        if self.__kopf == None:
            self.__schwanz = None
        return kopfObjekt
        
    def DerNaechsteIst (self) :
        '''gibt den Inhalt am Kopf der Warteschlange zurück'''
        if self.__kopf == None:
            return None
        return self.__kopf.GibInhalt()
        
    def GibLaenge (self) :
        '''gibt die aktuelle Länge der Warteschlange zurück'''
        laenge = 0
        # Lösung ohne Iterator
##        aktuell = self.__kopf
##        while aktuell!=None:
##            laenge+=1
##            aktuell = aktuell.GibNachfolger()
        # Lösung mit Iterator
        for element in self:
            laenge += 1
        return laenge
        
    def IstLeer (self) :
        '''gibt zurück, ob die Warteschlange leer ist'''
        return (self.__kopf == None)
    
    def Letzter(self):
        '''gibt das letzte Element der Warteschlange zurück'''
        if not self.IstLeer():
            return self.__schwanz.GibInhalt()
        return None
    
## --------------------------------------------
class Element_WS(object) :
    '''Klasse für ein Glied einer Warteschlange
    Der Nachfolger ist bei einer WS immer None.
    Der Inhalt kann nachträglich nicht gesetzt werden.'''
    def __init__(self, inhalt) :
        self.__inhalt = inhalt # object
        self.__naechstes = None # Element_WS
        
    def SetzeNachfolger (self, nachfolger) :
        self.__naechstes = nachfolger
        
    def GibInhalt (self) :
        return self.__inhalt
        
    def GibNachfolger (self) :
        return self.__naechstes
        
### ------------- Tests ------------------
import unittest

class TestWarteschlange(unittest.TestCase):

    def setUp(self):
        self.warteschlange = Warteschlange()
    
    def test_Initialisierung(self):
        self.assertTrue(self.warteschlange.IstLeer())
        self.assertEqual(self.warteschlange.GibLaenge(), 0)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), None)
        self.assertEqual(self.warteschlange.Letzter(), None)
        # Problem Aufrufen
        self.assertEqual(self.warteschlange.Aufrufen(), None)

    def test_Aktion(self):
        self.warteschlange.Anstellen('Hund')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==1)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Hund')
        self.assertEqual(self.warteschlange.Letzter(), 'Hund')
        
        self.warteschlange.Anstellen('Katze')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==2)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Hund')
        self.assertEqual(self.warteschlange.Letzter(), 'Katze')
        
        self.warteschlange.Anstellen('Vogel')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==3)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Hund')
        self.assertEqual(self.warteschlange.Letzter(), 'Vogel')
        
        self.assertEqual(self.warteschlange.Aufrufen(), 'Hund')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==2)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Katze')
        self.assertEqual(self.warteschlange.Letzter(), 'Vogel')
        
        self.assertEqual(self.warteschlange.Aufrufen(), 'Katze')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==1)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Vogel')
        self.assertEqual(self.warteschlange.Letzter(), 'Vogel')
        
        self.assertEqual(self.warteschlange.Aufrufen(), 'Vogel')
        self.assertTrue(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==0)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), None)
        self.assertEqual(self.warteschlange.Letzter(), None)

        # Problem: Was liefert Aufrufen jetzt?
        self.assertEqual(self.warteschlange.Aufrufen(), None)

    def test_Iteration(self):
        self.warteschlange.Anstellen('Hund')
        self.warteschlange.Anstellen('Katze')
        self.warteschlange.Anstellen('Vogel')
        ausgabe = []
        for element in self.warteschlange:
            ausgabe.append(element)
        self.assertEqual(ausgabe, ['Hund', 'Katze', 'Vogel'])

if __name__ == '__main__':
    unittest.main()
