# -*- coding: utf-8 -*-
# Warteschlange_mit_Iterator_Typ_Liste.py
'''Das Projekt implementiert objektorientiert
eine Warteschlange mit einem Iterator,
intern realisiert mit einer einfachen Python-Liste'''

class Warteschlange(object) :
    '''Implementiert eine Warteschlange mit Iterator'''

    def __init__(self) :
        '''Konstruktor'''
        self.__inhalte = []

    def __iter__(self):
        self.index = 0
        return self

    def __next__(self):
        if self.index>=len(self.__inhalte):
            raise StopIteration
        else:
            wert=self.__inhalte[self.index]
            self.index += 1
            return wert
            
    def Anstellen (self, objekt) :
        '''hängt ein neues Element am Ende der Warteschlange an'''
        self.__inhalte.append(objekt)

    def Aufrufen (self) :
        '''setzt den Kopf der Warteschlange weiter und
        gibt den Inhalt des bisherigen Kopfes zurück'''
        if len(self.__inhalte)==0:
            return None
        kopfObjekt = self.__inhalte[0]
        self.__inhalte=self.__inhalte[1:]
        return kopfObjekt
        
    def DerNaechsteIst (self) :
        '''gibt den Inhalt am Kopf der Warteschlange zurück'''
        if len(self.__inhalte)==0:
            return None
        return self.__inhalte[0]
        
    def GibLaenge (self) :
        '''gibt die aktuelle Länge der Warteschlange zurück'''
        return len(self.__inhalte)
        
    def IstLeer (self) :
        '''gibt zurück, ob die Warteschlange leer ist'''
        return (len(self.__inhalte) == 0)
    
    def Letzter(self):
        '''gibt das letzte Element der Warteschlange zurück'''
        if len(self.__inhalte)!=0:
            return self.__inhalte[len(self.__inhalte)-1]
        return None
    
        
### ------------- Tests ------------------
import unittest

class TestWarteschlange(unittest.TestCase):

    def setUp(self):
        self.warteschlange = Warteschlange()
    
    def test_Initialisierung(self):
        self.assertTrue(self.warteschlange.IstLeer())
        self.assertEqual(self.warteschlange.GibLaenge(), 0)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), None)
        self.assertEqual(self.warteschlange.Letzter(), None)
        # Problem Aufrufen
        self.assertEqual(self.warteschlange.Aufrufen(), None)

    def test_Aktion(self):
        self.warteschlange.Anstellen('Hund')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==1)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Hund')
        self.assertEqual(self.warteschlange.Letzter(), 'Hund')
        
        self.warteschlange.Anstellen('Katze')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==2)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Hund')
        self.assertEqual(self.warteschlange.Letzter(), 'Katze')
        
        self.warteschlange.Anstellen('Vogel')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==3)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Hund')
        self.assertEqual(self.warteschlange.Letzter(), 'Vogel')
        
        self.assertEqual(self.warteschlange.Aufrufen(), 'Hund')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==2)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Katze')
        self.assertEqual(self.warteschlange.Letzter(), 'Vogel')
        
        self.assertEqual(self.warteschlange.Aufrufen(), 'Katze')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==1)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Vogel')
        self.assertEqual(self.warteschlange.Letzter(), 'Vogel')
        
        self.assertEqual(self.warteschlange.Aufrufen(), 'Vogel')
        self.assertTrue(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==0)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), None)
        self.assertEqual(self.warteschlange.Letzter(), None)

        # Problem: Was liefert Aufrufen jetzt?
        self.assertEqual(self.warteschlange.Aufrufen(), None)

    def test_Iteration(self):
        self.warteschlange.Anstellen('Hund')
        self.warteschlange.Anstellen('Katze')
        self.warteschlange.Anstellen('Vogel')
        ausgabe = []
        for element in self.warteschlange:
            ausgabe.append(element)
        self.assertEqual(ausgabe, ['Hund', 'Katze', 'Vogel'])

if __name__ == '__main__':
    unittest.main()
