# -*- coding: utf-8 -*-
# Warteschlange_mit_Iterator_Typ_Tupel.py
'''Das Projekt implementiert objektorientiert
eine Warteschlange mit einem Iterator,
intern realisiert mit Tupeln
Es muss ein 'tiefes Tupel' aufgebaut werden
-> Rekursion!'''

class Warteschlange(object) :
    '''Implementiert eine Warteschlange mit Iterator'''

    def __init__(self) :
        '''Konstruktor'''
        self.__kopf = None

    def __iter__(self):
        '''initialisiert den Iterator'''
        self.aktuell=self.__kopf
        return self

    def __next__(self):
        '''definiert den Iterationsschritt'''
        if self.aktuell==None:
            raise StopIteration
        else:
            wert=self.aktuell[0]
            self.aktuell=self.aktuell[1]
            return wert
            
    def Anstellen (self, objekt) :
        '''hängt ein neues Element am Ende der Warteschlange an'''
        if self.__kopf == None:
            self.__kopf = (objekt, None)
        else:
            self.__kopf = self.Anhaengen (self.__kopf, objekt)
            
    def Anhaengen(self, aktuell, objekt):
        if aktuell[1]==None:
            return (aktuell[0], (objekt, None))
        else:
            return (aktuell[0], self.Anhaengen(aktuell[1], objekt))
        

    def Aufrufen (self) :
        '''setzt den Kopf der Warteschlange weiter und
        gibt den Inhalt des bisherigen Kopfes zurück'''
        if self.__kopf == None:
            return None
        kopfObjekt = self.__kopf[0]
        self.__kopf = self.__kopf[1]
        return kopfObjekt
        
    def DerNaechsteIst (self) :
        '''gibt den Inhalt am Kopf der Warteschlange zurück'''
        if self.__kopf == None:
            return None
        return self.__kopf[0]
        
    def GibLaenge (self) :
        '''gibt die aktuelle Länge der Warteschlange zurück'''
        laenge = 0
        # Lösung mit Iterator
        for element in self:
            laenge += 1
        return laenge
        
    def IstLeer (self) :
        '''gibt zurück, ob die Warteschlange leer ist'''
        return (self.__kopf == None)
    
    def Letzter(self):
        '''gibt den Inhalt des letzten Elements der Warteschlange zurück'''
        ergebnis=None
        for element in self:
            if element!=None:
                ergebnis=element
        return ergebnis
            
    
### ------------- Tests ------------------
import unittest

class TestWarteschlange(unittest.TestCase):

    def setUp(self):
        self.warteschlange = Warteschlange()
    
    def test_Initialisierung(self):
        self.assertTrue(self.warteschlange.IstLeer())
        self.assertEqual(self.warteschlange.GibLaenge(), 0)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), None)
        self.assertEqual(self.warteschlange.Letzter(), None)
        # Problem Aufrufen
        self.assertEqual(self.warteschlange.Aufrufen(), None)

    def test_Aktion(self):
        self.warteschlange.Anstellen('Hund')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==1)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Hund')
        self.assertEqual(self.warteschlange.Letzter(), 'Hund')
        
        self.warteschlange.Anstellen('Katze')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==2)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Hund')
        self.assertEqual(self.warteschlange.Letzter(), 'Katze')
        
        self.warteschlange.Anstellen('Vogel')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==3)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Hund')
        self.assertEqual(self.warteschlange.Letzter(), 'Vogel')
        
        self.assertEqual(self.warteschlange.Aufrufen(), 'Hund')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==2)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Katze')
        self.assertEqual(self.warteschlange.Letzter(), 'Vogel')
        
        self.assertEqual(self.warteschlange.Aufrufen(), 'Katze')
        self.assertFalse(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==1)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), 'Vogel')
        self.assertEqual(self.warteschlange.Letzter(), 'Vogel')
        
        self.assertEqual(self.warteschlange.Aufrufen(), 'Vogel')
        self.assertTrue(self.warteschlange.IstLeer())
        self.assertTrue(self.warteschlange.GibLaenge()==0)
        self.assertEqual(self.warteschlange.DerNaechsteIst(), None)
        self.assertEqual(self.warteschlange.Letzter(), None)

        # Problem: Was liefert Aufrufen jetzt?
        self.assertEqual(self.warteschlange.Aufrufen(), None)

    def test_Iteration(self):
        self.warteschlange.Anstellen('Hund')
        self.warteschlange.Anstellen('Katze')
        self.warteschlange.Anstellen('Vogel')
        ausgabe = []
        for element in self.warteschlange:
            ausgabe.append(element)
        self.assertEqual(ausgabe, ['Hund', 'Katze', 'Vogel'])

if __name__ == '__main__':
    unittest.main()
